<?php

namespace SM\Service;

use SM\Factory\SM_NEXT_00006_000_servicosFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class SM_NEXT_00006_000_servicosService {

    protected $ServicoFactory;
    protected $Environment;
    protected $newCode;
    protected $procedureService;

    public function __construct (EntityManager $entityManager, SM_NEXT_00006_000_servicosFactory $ServicoFactory, Environment $Environment, NewCode $newCode) {
        $this->ServicoFactory = $ServicoFactory;
        $this->connection         = $entityManager->getConnection();
        $this->Environment        = $Environment;
        $this->newCode            = $newCode;
    }
    
    public function getInformacoesGridPrincipal() {
        return $this->ServicoFactory->getInformacoesGridPrincipal();
    }

    public function deletarServicos(array $codigos) {
        $stringCodigos = implode(",", $codigos);
        $this->ServicoFactory->deletarServicos($stringCodigos);
    }

    public function getProdutoFaturamento() {
        return $this->ServicoFactory->getProdutoFaturamento();
    }
    
    public function findHabilidades($filtro) {
        return $this->ServicoFactory->findHabilidades($filtro);
    }

    public function getRelacServico($filtro) {
        return $this->ServicoFactory->getRelacServico($filtro);
    }

    public function removeRelacaoHabilidade($params) {
        foreach ($params as $param){
            $data = array(
                "NRRELACSERVHABILI" => $param["NRRELACSERVHABILI"],
            );
            $this->ServicoFactory->removeRelacaoHabilidade($data);
        }
    }

    public function validacao($servico, $habilidade, $nrorg) {
        $habilidade = implode(",",$habilidade);
        $dados = $this->ServicoFactory-> validacao($servico, $habilidade, $nrorg);
        $resultado = $dados[0]["EXISTE"];
        return ($resultado == 0);
    }
    
    public function insertServico($idativo, $nmservorca, $cdprodfatserv, $dsservorca, $dsservorcanf, $dsservorcabol, $cdservorca) {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrOrg();

            if ($cdservorca == null) {
                $codigo = $this->newCode->getCodeByTableName("FCT_SERVICOORCA", $nrorg);
    
                $parametros = array(
                    "CDSERVORCA" => $codigo,
                    "NRORG" => $nrorg,
                    "IDATIVO" => $idativo,
                    "CDOPERINCLUSAO" => $this->Environment->getCdOperador(),
                    "NMSERVORCA" => $nmservorca,
                    "CDPRODFATSERV" => $cdprodfatserv,
                    "DSSERVORCA" => $dsservorca,
                    "DSSERVORCANF" => $dsservorcanf,
                    "DSSERVORCABOL" => $dsservorcabol,
                    "CDOPERULTATU" => $this->Environment->getCdOperador(),
                );
                $this->ServicoFactory->adicionarServico($parametros);
    
            } else {
                $parametros = array (
                    "CDSERVORCA" => $cdservorca,
                    "IDATIVO" => $idativo,
                    "NMSERVORCA" => $nmservorca,
                    "CDPRODFATSERV" => $cdprodfatserv,
                    "DSSERVORCA" => $dsservorca,
                    "DSSERVORCANF" => $dsservorcanf,
                    "DSSERVORCABOL" => $dsservorcabol,
                    "NRORGULTATU" => $nrorg,
                    "CDOPERULTATU" => $this->Environment->getCdOperador(),
                );
                $this->ServicoFactory->editarServico($parametros);
            }

            $this->connection->commit();

        }  catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function insertRelacaoHabilidade($servico, $habilidades, $nrorg) {
        foreach($habilidades as $habilidade){
            $codigo = $this->newCode->getCodeByTableName("FCT_RELACSERVHABILI", $nrorg, 1, 10, true);
            $parametros = array(
                "NRRELACSERVHABILI" => $codigo,
                "CDSERVORCA" => $servico,
                "CDHABILITSERV" => $habilidade,
                "NRORG" => $nrorg,
                "IDATIVO" => 'S',
                "NRORGINCLUSAO" => $nrorg,
                "NRORGULTATU" => $nrorg,
                "CDOPERULTATU" => $this->Environment->getCdOperador(),
            );
            $this->ServicoFactory->insertRelacaoHabilidade($parametros);
        }
    }

    public function insertRelacServico($params) {
        $nrorg = $this->Environment->getNrOrg();
        foreach($params['CDSERVORCARELAC'] as $servicoRelac){
            $codigo = $this->newCode->getCodeByTableName("FCT_RELACSERVICO", $nrorg, 1, 10, true);
            $parametros = array(
                "NRRELACSERVICO" => $codigo,
                "CDSERVORCA" => $params['CDSERVORCA'],
                "CDSERVORCARELAC" => $servicoRelac,
                "CDTPMEDIDASERV" => $params['CDTPMEDIDASERV'],
                "QTMEDIDASERV" => $params['QTMEDIDASERV'],
                "NRORG" => $nrorg,
                "IDATIVO" => 'S',
                "NRORGINCLUSAO" => $nrorg,
                "CDOPERINCLUSAO" => $codigo,
                "NRORGULTATU" => $nrorg,
                "CDOPERULTATU" => $this->Environment->getCdOperador(),
            );
            $this->ServicoFactory->insertRelacServico($parametros);
        }
    }

    public function validaServicoRelacionado($servOrcaRelac, $servOrca, $tipoMedida){
    
        $paramsValidaServRelac = array(
            "CDSERVORCARELAC"  => implode(',', $servOrcaRelac),
            "CDSERVORCA"       => $servOrca,
            "CDTPMEDIDASERV"   => $tipoMedida,
            "NRORG"            => $this->Environment->getNrOrg()
        );
        
        $returnValidaServRelac = $this->ServicoFactory->validaServicoRelacionado($paramsValidaServRelac);
        
        return $returnValidaServRelac[0]["VALIDASERV"] != 0;
        
    }

    public function deleteRelacServico($params) {
        foreach($params as $relac){
            $NRRELACSERVICO = array(
                "NRRELACSERVICO" => $relac['NRRELACSERVICO']
            );
            $this->ServicoFactory->deleteRelacServico($NRRELACSERVICO);
        }
    }
}